@@ -12,8 +12,9 @@ from kazoo.client import KazooClient
 from kazoo.exceptions import NoAuthError, NoNodeError
 from kazoo.protocol.states import KazooState
 
+from .tree import Tree
 from .usage import Usage
-from .util import to_bytes
+from .util import join, to_bytes
 
 
 @contextmanager
@@ -197,28 +198,9 @@ class AugumentedClient(KazooClient):
             print("Bad regexp: %s" % (ex))
             return
 
-        for fpath in self.do_find(path, match, True):
-            yield fpath
-
-    def do_find(self, path, match, check_match):
-        """ find's work horse """
-        try:
-            children = self.get_children(path)
-        except (NoNodeError, NoAuthError):
-            children = []
-
-        for child in children:
-            check = check_match
-            full_path = os.path.join(path, child)
-            if check:
-                if match.search(full_path):
-                    yield full_path
-                    check = False
-            else:
-                yield full_path
-
-            for fpath in self.do_find(full_path, match, check):
-                yield fpath
+        for cpath in Tree(self, path).get():
+            if match.search(cpath):
+                yield cpath
 
     def grep(self, path, content, flags):
         """ grep every child path under path for content """
@@ -283,18 +265,22 @@ class AugumentedClient(KazooClient):
             children = []
 
         for child in children:
-            cpath = u"%s/%s" % (path.rstrip("/"), child) if full_path else child
+            cpath = join(path, child) if full_path else child
             if include_stat:
-                stat_path = str(u"%s/%s" % (path.rstrip("/"), child))
-                yield cpath, level, self.stat(stat_path)
+                yield cpath, level, self.stat(join(path, child))
             else:
                 yield cpath, level
 
             if max_depth == 0 or level + 1 < max_depth:
-                cpath = u"%s/%s" % (path.rstrip("/"), child)
+                cpath = join(path, child)
                 for rchild_rlevel_rstat in self.do_tree(cpath, max_depth, level + 1, full_path, include_stat):
                     yield rchild_rlevel_rstat
 
+    def fast_tree(self, path):
+        """ a fast async version of tree() """
+        for cpath in Tree(self, path).get():
+            yield cpath
+
     def diff(self, path_a, path_b):
         """ Performs a deep comparison of path_a/ and path_b/
 
@@ -320,7 +306,7 @@ class AugumentedClient(KazooClient):
         # first, check what's missing & changed in dst
         for child_a, level in self.tree(path_a, 0, True):
             child_sub = child_a[len_a + 1:]
-            child_b = "%s/%s" % (path_b, child_sub)
+            child_b = join(path_b, child_sub)
 
             if not self.exists(child_b):
                 yield -1, child_sub
