@@ -27,6 +27,7 @@ It supports the basic ops plus a few handy extensions:
 
 from __future__ import print_function
 
+from collections import defaultdict
 from contextlib import contextmanager
 from functools import partial, wraps
 import json
@@ -69,17 +70,20 @@ from .augumented_cmd import (
 )
 from .copy import CopyError, Proxy
 from .keys import Keys
+from .pathmap import PathMap
 from .watcher import get_child_watcher
 from .watch_manager import get_watch_manager
 from .util import (
+    decoded,
+    join,
+    invalid_hosts,
     Netloc,
     pretty_bytes,
+    prompt_yes_no,
+    split,
     to_bool,
     to_float,
     to_int,
-    decoded,
-    prompt_yes_no,
-    invalid_hosts
 )
 
 
@@ -152,6 +156,21 @@ def default_watcher(watched_event):
 HISTORY_FILENAME = ".kz-shell-history"
 
 
+class BadJSON(Exception): pass
+
+
+def json_deserialize(data):
+    if data is None:
+        raise BadJSON(path)
+
+    try:
+        obj = json.loads(data)
+    except ValueError:
+        raise BadJSON(path)
+
+    return obj
+
+
 # pylint: disable=R0904
 class Shell(AugumentedCmd):
     """ main class """
@@ -188,7 +207,7 @@ class Shell(AugumentedCmd):
 
         if self._zk.exists(path):
             children = self._zk.get_children(self.resolve_path(path))
-            opts = ["%s/%s" % (path, znode) for znode in children]
+            opts = [join(path, znode) for znode in children]
         elif "/" not in path:
             znodes = self._zk.get_children(self.curdir)
             opts = [znode for znode in znodes if znode.startswith(path)]
@@ -196,7 +215,7 @@ class Shell(AugumentedCmd):
             parent = os.path.dirname(path)
             child = os.path.basename(path)
             matching = [znode for znode in self._zk.get_children(parent) if znode.startswith(child)]
-            opts = ["%s/%s" % (parent, znode) for znode in matching]
+            opts = [join(parent, znode) for znode in matching]
 
         return [opt[offs:] for opt in opts]
 
@@ -501,6 +520,50 @@ class Shell(AugumentedCmd):
 
     complete_find = _complete_path
 
+    @connected
+    @ensure_params(
+        Required("path"),
+        Required("pattern"),
+        BooleanOptional("inverse", default=False)
+    )
+    @check_paths_exists("path")
+    def do_child_matches(self, params):
+        """
+        Prints paths that have at least 1 child that matches <pattern>
+
+        child_matches <path> <pattern> [inverse]
+
+        Example:
+
+        child_matches /services/registrations member_
+        /services/registrations/foo
+        /services/registrations/bar
+        ...
+
+        Output can be inverted (inverse = true) to display all paths that don't have children matching
+        the given pattern.
+
+        """
+        seen = set()
+
+        for path in self._zk.fast_tree(params.path):
+            parent, child = split(path)
+
+            if parent in seen:
+                continue
+
+            match = params.pattern in child
+            if params.inverse:
+                if not match:
+                    self.show_output(parent)
+                    seen.add(parent)
+            else:
+                if match:
+                    self.show_output(parent)
+                    seen.add(parent)
+
+    complete_child_matches = _complete_path
+
     @connected
     @ensure_params(Optional("path"), Required("match"))
     @check_paths_exists("path")
@@ -911,6 +974,8 @@ child_watches=%s"""
         if count == 0:
             self.show_output("Branches are equal.")
 
+    complete_diff = _complete_path
+
     @connected
     @ensure_params(Required("path"), BooleanOptional("recursive"))
     @check_paths_exists("path")
@@ -1040,26 +1105,6 @@ child_watches=%s"""
             self.show_output(str(ex))
             return
 
-        class BadJSON(Exception): pass
-
-        def get(path, keys, print_path):
-            jstr, _ = self._zk.get(path)
-
-            if jstr is None:
-                raise BadJSON(path)
-
-            try:
-                obj = json.loads(jstr)
-            except ValueError:
-                raise BadJSON(path)
-
-            value = Keys.value(obj, keys)
-
-            if print_path:
-                self.show_output("%s: %s", os.path.basename(path), value)
-            else:
-                self.show_output(value)
-
         if params.recursive:
             paths = self._zk.tree(params.path, 0, full_path=True)
             print_path = True
@@ -1069,7 +1114,13 @@ child_watches=%s"""
 
         for cpath, _ in paths:
             try:
-                get(cpath, params.keys, print_path)
+                jstr, _ = self._zk.get(cpath)
+                value = Keys.value(json_deserialize(jstr), params.keys)
+
+                if print_path:
+                    self.show_output("%s: %s", os.path.basename(cpath), value)
+                else:
+                    self.show_output(value)
             except BadJSON as ex:
                 self.show_output("Path %s has bad JSON.", cpath)
             except Keys.Missing as ex:
@@ -1077,6 +1128,69 @@ child_watches=%s"""
 
     complete_json_get = _complete_path
 
+    @connected
+    @ensure_params(
+        Required("path"),
+        Required("keys"),
+        IntegerOptional("top", 0),
+        IntegerOptional("minfreq", 1),
+        BooleanOptional("reverse", default=True)
+    )
+    @check_paths_exists("path")
+    def do_json_count_values(self, params):
+        """
+        Counts the frequency of values associated with <keys>, for all JSON dicts stored in <path>'s children
+
+        json_count_values <path> <keys> [top] [minfreq] [reverse]
+
+        Example:
+
+        json_count_values /configs/primary_service endpoint.host
+        10.20.0.2  3
+        10.20.0.4  3
+        10.20.0.5  3
+        10.20.0.6  1
+        10.20.0.7  1
+        ...
+
+        By default, all values are shown (top = 0) regardless of their frequency (minfreq = 1).
+        They are sorted by frequency in descendant order (reverse = true).
+
+        """
+        try:
+            Keys.validate(params.keys)
+        except Keys.Bad as ex:
+            self.show_output(str(ex))
+            return
+
+        path_map = PathMap(self._zk, params.path)
+
+        values = defaultdict(int)
+        for path, data in path_map.get():
+            try:
+                value = Keys.value(json_deserialize(data), params.keys)
+                values[value] += 1
+            except BadJSON as ex:
+                self.show_output("Path %s has bad JSON.", path)
+            except Keys.Missing as ex:
+                self.show_output("Path %s is missing key %s.", path, ex)
+
+        results = sorted(values.items(), key=lambda item: item[1], reverse=params.reverse)
+
+        i = 0
+        for value, frequency in results:
+            if frequency < params.minfreq:
+                continue
+
+            self.show_output("%s = %d", value, frequency)
+
+            if params.top > 0 and i > params.top:
+                break
+
+            i += 1
+
+    complete_json_count_values = _complete_path
+
     @contextmanager
     def transitions_disabled(self):
         """
